!-- MeasuredValue_Form handles numbers with labels (typically units) providing
!   means of dealing with units, for example. 

module MeasuredValue_Form
  
  use KIND_DEFAULT_Singleton
  use LEN_DEFAULT_Singleton
  use Split_Command
  use Join_Command
  
  implicit none
  private
  
  type, public :: MeasuredValueForm
    real ( KDR ) :: &
      Number = 1.0_KDR
    character ( LDL ) :: &
      Unit = ''
    character ( LDL ) :: &
      Label = '' !-- Actual unit name if this object is being used as a unit
  contains
    procedure, private, pass :: &
      Initialize_MV
    procedure, private, pass :: &
      Initialize_MV_Label
    procedure, private, pass :: &
      Initialize_MV_From_MV
    procedure, private, pass :: &
      Initialize_MV_From_MV_Label
    generic :: &
      Initialize &
        => Initialize_MV, Initialize_MV_Label, Initialize_MV_From_MV, &
           Initialize_MV_From_MV_Label
    procedure, private, pass :: &
      Addition_MV_MV
    procedure, private, pass :: &
      Addition_MV_Real
    procedure, private, pass ( Addend ) :: &
      AdditionReal_MV
    procedure, private, pass :: &
      Addition_MV_Integer
    procedure, private, pass ( Addend ) :: &
      AdditionInteger_MV
    generic :: &
      operator ( + ) &
        => Addition_MV_MV, Addition_MV_Real, AdditionReal_MV, &
           Addition_MV_Integer, AdditionInteger_MV
    procedure, private, pass :: &
      Subtraction_MV_MV
    procedure, private, pass :: &
      Subtraction_MV_Real
    procedure, private, pass ( Subtrahend ) :: &
      SubtractionReal_MV
    procedure, private, pass :: &
      Subtraction_MV_Integer
    procedure, private, pass ( Subtrahend ) :: &
      SubtractionInteger_MV
    generic :: &
      operator ( - ) &
        => Subtraction_MV_MV, Subtraction_MV_Real, SubtractionReal_MV, &
           Subtraction_MV_Integer, SubtractionInteger_MV
    procedure, private, pass :: &
      Product_MV_MV
    procedure, private, pass :: &
      Product_MV_Real
    procedure, private, pass ( Multiplicand ) :: &
      ProductReal_MV
    procedure, private, pass :: &
      Product_MV_Integer
    procedure, private, pass ( Multiplicand ) :: &
      ProductInteger_MV
    generic :: &
      operator ( * ) &
        => Product_MV_MV, Product_MV_Real, ProductReal_MV, &
           Product_MV_Integer, ProductInteger_MV
    procedure, private, pass :: &
      Power_MV_Integer
    procedure, private, pass :: &
      Power_MV_Real
    generic :: &
      operator ( ** ) => Power_MV_Integer, Power_MV_Real
    procedure, private, pass :: &
      Quotient_MV_MV
    procedure, private, pass :: &
      Quotient_MV_Real
    procedure, private, pass ( Divisor ) :: &
      QuotientReal_MV
    procedure, private, pass :: &
      Quotient_MV_Integer
    procedure, private, pass ( Divisor ) :: &
      QuotientInteger_MV
    generic :: &
      operator ( / ) &
        => Quotient_MV_MV, Quotient_MV_Real, QuotientReal_MV, &
           Quotient_MV_Integer, QuotientInteger_MV
    procedure, private, pass ( MV ) :: &
      AssignReal_MV
    generic :: &
      assignment ( = ) => AssignReal_MV
    procedure, private, pass :: &
      EqualTo_MV_MV
    procedure, private, pass :: &
      EqualTo_MV_Real
    procedure, private, pass ( MV ) :: &
      EqualToReal_MV
    procedure, private, pass :: &
      EqualTo_MV_Integer
    procedure, private, pass ( MV ) :: &
      EqualToInteger_MV
    generic :: &
      operator ( == ) &
        => EqualTo_MV_MV, EqualTo_MV_Real, EqualToReal_MV, &
           EqualTo_MV_Integer, EqualToInteger_MV
    procedure, private, pass :: &
      NotEqualTo_MV_MV
    procedure, private, pass :: &
      NotEqualTo_MV_Real
    procedure, private, pass ( MV ) :: &
      NotEqualToReal_MV
    procedure, private, pass :: &
      NotEqualTo_MV_Integer
    procedure, private, pass ( MV ) :: &
      NotEqualToInteger_MV
    generic :: &
      operator ( /= ) &
        => NotEqualTo_MV_MV, NotEqualTo_MV_Real, NotEqualToReal_MV, &
           NotEqualTo_MV_Integer, NotEqualToInteger_MV
    procedure, private, pass :: &
      GreaterThan_MV_MV
    procedure, private, pass :: &
      GreaterThan_MV_Real
    procedure, private, pass ( MV ) :: &
      GreaterThanReal_MV
    procedure, private, pass :: &
      GreaterThan_MV_Integer
    procedure, private, pass ( MV ) :: &
      GreaterThanInteger_MV
    generic :: &
      operator ( > ) &
        => GreaterThan_MV_MV, GreaterThan_MV_Real, GreaterThanReal_MV, &
           GreaterThan_MV_Integer, GreaterThanInteger_MV
    procedure, private, pass :: &
      LessThan_MV_MV
    procedure, private, pass :: &
      LessThan_MV_Real
    procedure, private, pass ( MV ) :: &
      LessThanReal_MV
    procedure, private, pass :: &
      LessThan_MV_Integer
    procedure, private, pass ( MV ) :: &
      LessThanInteger_MV
    generic :: &
      operator ( < ) &
        => LessThan_MV_MV, LessThan_MV_Real, LessThanReal_MV, &
           LessThan_MV_Integer, LessThanInteger_MV
    procedure, private, pass :: &
      GreaterThanEqualTo_MV_MV
    procedure, private, pass :: &
      GreaterThanEqualTo_MV_Real
    procedure, private, pass ( MV ) :: &
      GreaterThanEqualToReal_MV
    procedure, private, pass :: &
      GreaterThanEqualTo_MV_Integer
    procedure, private, pass ( MV ) :: &
      GreaterThanEqualToInteger_MV
    generic :: &
      operator ( >= ) &
        => GreaterThanEqualTo_MV_MV, GreaterThanEqualTo_MV_Real, &
           GreaterThanEqualToReal_MV, GreaterThanEqualTo_MV_Integer, &
           GreaterThanEqualToInteger_MV
    procedure, private, pass :: &
      LessThanEqualTo_MV_MV
    procedure, private, pass :: &
      LessThanEqualTo_MV_Real
    procedure, private, pass ( MV ) :: &
      LessThanEqualToReal_MV
    procedure, private, pass :: &
      LessThanEqualTo_MV_Integer
    procedure, private, pass ( MV ) :: &
      LessThanEqualToInteger_MV
    generic :: &
      operator ( <= ) &
        => LessThanEqualTo_MV_MV, LessThanEqualTo_MV_Real, &
           LessThanEqualToReal_MV, LessThanEqualTo_MV_Integer, &
           LessThanEqualToInteger_MV
    procedure, private, nopass :: &
      ProductUnit
    procedure, private, nopass :: &
      PowerUnitInteger
    procedure, private, nopass :: &
      PowerUnitReal
    generic :: &
      PowerUnit => PowerUnitInteger, PowerUnitReal
  end type MeasuredValueForm

contains


  subroutine Initialize_MV ( MV, Unit, Number )
    
    class ( MeasuredValueForm ), intent ( inout ) :: &
      MV
    character ( * ), intent ( in ) :: &
      Unit
    real ( KDR ), intent ( in ) :: &
      Number
      
    MV % Number = Number
    MV % Unit   = Unit

  end subroutine Initialize_MV
  
  
  subroutine Initialize_MV_Label ( MV, Label, Unit, Number )
    
    class ( MeasuredValueForm ), intent ( inout ) :: &
      MV
    character ( * ), intent ( in ) :: &
      Label, &
      Unit
    real ( KDR ), intent ( in ) :: &
      Number
      
    MV % Number = Number
    MV % Unit   = Unit
    MV % Label  = Label

  end subroutine Initialize_MV_Label
  
  
  subroutine Initialize_MV_From_MV ( MV_Target, MV_Source )
    
    class ( MeasuredValueForm ), intent ( inout ) :: &
      MV_Target
    type ( MeasuredValueForm ), intent ( in ) :: &
      MV_Source
            
    MV_Target % Number = MV_Source % Number
    MV_Target % Unit   = MV_Source % Unit
    MV_Target % Label  = MV_Source % Label
  
  end subroutine Initialize_MV_From_MV
  
  
  subroutine Initialize_MV_From_MV_Label ( MV_Target, MV_Source, Label )
    
    class ( MeasuredValueForm ), intent ( inout ) :: &
      MV_Target
    type ( MeasuredValueForm ), intent ( in ) :: &
      MV_Source
    character ( * ), intent ( in ) :: &
      Label
            
    MV_Target % Number = MV_Source % Number
    MV_Target % Unit   = MV_Source % Unit
    MV_Target % Label  = Label
  
  end subroutine Initialize_MV_From_MV_Label
  
  
  elemental function Addition_MV_MV ( Augend, Addend ) result ( A_MV_MV )

    class ( MeasuredValueForm ), intent ( in ) :: &
      Augend
    type ( MeasuredValueForm ), intent ( in ) :: &
      Addend
    type ( MeasuredValueForm ) :: &
      A_MV_MV
    
      A_MV_MV % Number = Augend % Number + Addend % Number
    
      if ( trim ( Augend % Unit ) == trim ( Addend % Unit ) ) then
        A_MV_MV % Unit = Augend % Unit
      else
        A_MV_MV % Unit = 'Undefined'
      end if
  
      if ( trim ( Augend % Label ) == trim ( Addend % Label ) ) then
        A_MV_MV % Label = Augend % Label
      else
        A_MV_MV % Label = 'Undefined'
      end if
  
  end function Addition_MV_MV
  
  
  elemental function Addition_MV_Real &
                       ( Augend, Addend ) result ( A_MV_R )
  
    class ( MeasuredValueForm ), intent ( in ) :: &
      Augend
    real ( KDR ), intent ( in ) :: &
      Addend
    real ( KDR ) :: &
      A_MV_R
      
    A_MV_R = Augend % Number + Addend
      
  end function Addition_MV_Real
   
  
  elemental function AdditionReal_MV &
                       ( Augend, Addend ) result ( A_R_MV )
  
    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "Augend + Addend" 

    real ( KDR ), intent ( in ) :: &
      Augend
    class ( MeasuredValueForm ), intent ( in ) :: &
      Addend
    real ( KDR ) :: &
      A_R_MV
      
    A_R_MV = Augend + Addend % Number
      
  end function AdditionReal_MV
   
  
  elemental function Addition_MV_Integer &
                       ( Augend, Addend ) result ( A_MV_I )
  
    class ( MeasuredValueForm ), intent ( in ) :: &
      Augend
    integer ( KDI ), intent ( in ) :: &
      Addend
    real ( KDR ) :: &
      A_MV_I
      
    A_MV_I = Augend % Number + real ( Addend, KDR )
      
  end function Addition_MV_Integer
   
  
  elemental function AdditionInteger_MV &
                       ( Augend, Addend ) result ( A_I_MV )
  
    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "Augend + Addend" 

    integer ( KDI ), intent ( in ) :: &
      Augend
    class ( MeasuredValueForm ), intent ( in ) :: &
      Addend
    real ( KDR ) :: &
      A_I_MV
      
    A_I_MV = real ( Augend, KDR ) + Addend % Number
      
  end function AdditionInteger_MV
   
  
  elemental function Subtraction_MV_MV &
                       ( Minuend, Subtrahend ) result ( S_MV_MV )
  
    class ( MeasuredValueForm ), intent ( in ) :: &
      Minuend
    class ( MeasuredValueForm ), intent ( in ) :: &
      Subtrahend
    type ( MeasuredValueForm ) :: &
      S_MV_MV
      
    S_MV_MV % Number = Minuend % Number - Subtrahend % Number
      
    if ( trim ( Minuend % Unit ) == trim ( Subtrahend % Unit ) ) then
      S_MV_MV % Unit = Minuend % Unit
    else
      S_MV_MV % Unit = 'Undefined'
    end if
    
    if ( trim ( Minuend % Label ) == trim ( Subtrahend % Label ) ) then
      S_MV_MV % Label = Minuend % Label
    else
      S_MV_MV % Label = 'Undefined'
    end if
    
  end function Subtraction_MV_MV
  
  
  elemental function Subtraction_MV_Real &
                       ( Minuend, Subtrahend ) result ( S_MV_R )
  
    class ( MeasuredValueForm ), intent ( in ) :: &
      Minuend
    real ( KDR ), intent ( in ) :: &
      Subtrahend
    real ( KDR ) :: &
      S_MV_R
      
    S_MV_R = Minuend % Number - Subtrahend
      
  end function Subtraction_MV_Real
   
  
  elemental function SubtractionReal_MV &
                       ( Minuend, Subtrahend ) result ( S_R_MV )
  
    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "Minuend - Subtrahend" 

    real ( KDR ), intent ( in ) :: &
      Minuend
    class ( MeasuredValueForm ), intent ( in ) :: &
      Subtrahend
    real ( KDR ) :: &
      S_R_MV
      
    S_R_MV = Minuend - Subtrahend % Number
      
  end function SubtractionReal_MV
   
  
  elemental function Subtraction_MV_Integer &
                       ( Minuend, Subtrahend ) result ( S_MV_I )
  
    class ( MeasuredValueForm ), intent ( in ) :: &
      Minuend
    integer ( KDI ), intent ( in ) :: &
      Subtrahend
    real ( KDR ) :: &
      S_MV_I
      
    S_MV_I = Minuend % Number - real ( Subtrahend, KDR )
      
  end function Subtraction_MV_Integer
   
  
  elemental function SubtractionInteger_MV &
                       ( Minuend, Subtrahend ) result ( S_I_MV )
  
    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "Minuend - Subtrahend" 

    integer ( KDI ), intent ( in ) :: &
      Minuend
    class ( MeasuredValueForm ), intent ( in ) :: &
      Subtrahend
    real ( KDR ) :: &
      S_I_MV
      
    S_I_MV = real ( Minuend, KDR ) - Subtrahend % Number
      
  end function SubtractionInteger_MV
   
  
  elemental function Product_MV_MV &
              ( Multiplier, Multiplicand ) result ( P_MV_MV )
  
    class ( MeasuredValueForm ), intent ( in ) :: &
      Multiplier, &
      Multiplicand
    type ( MeasuredValueForm ) :: &
      P_MV_MV

    P_MV_MV % Number = Multiplier % Number * Multiplicand % Number
  
    P_MV_MV % Unit &
      = P_MV_MV % ProductUnit ( Multiplier % Unit, Multiplicand % Unit )

    P_MV_MV % Label &
      = P_MV_MV % ProductUnit ( Multiplier % Label, Multiplicand % Label )

  end function Product_MV_MV
  
  
  elemental function Product_MV_Real &
                       ( Multiplier, Multiplicand ) result ( P_MV_R )
    
    class ( MeasuredValueForm ), intent ( in ) :: &
      Multiplier
    real ( KDR ), intent ( in ) :: &
      Multiplicand
    type ( MeasuredValueForm ) :: &
      P_MV_R
    
    P_MV_R % Number = Multiplier % Number * Multiplicand
    P_MV_R % Unit   = Multiplier % Unit
    P_MV_R % Label  = Multiplier % Label
  
  end function Product_MV_Real
  

  elemental function ProductReal_MV &
                        ( Multiplier, Multiplicand ) result ( P_R_MV )
    
    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "Multiplier * Multiplicand" 

    real ( KDR ), intent ( in ) :: &
      Multiplier
    class ( MeasuredValueForm ), intent ( in ) :: &
      Multiplicand
    type ( MeasuredValueForm ) :: &
      P_R_MV
    
    P_R_MV % Number = Multiplier * Multiplicand % Number
    P_R_MV % Unit   = Multiplicand % Unit
    P_R_MV % Label  = Multiplicand % Label
    
  end function ProductReal_MV
  
  
  elemental function Product_MV_Integer &
                       ( Multiplier, Multiplicand ) result ( P_MV_I )
    
    class ( MeasuredValueForm ), intent ( in ) :: &
      Multiplier
    integer ( KDI ), intent ( in ) :: &
      Multiplicand
    type ( MeasuredValueForm ) :: &
      P_MV_I
    
    P_MV_I % Number = Multiplier % Number * real ( Multiplicand, KDR )
    P_MV_I % Unit   = Multiplier % Unit
    P_MV_I % Label  = Multiplier % Label
  
  end function Product_MV_Integer
  

  elemental function ProductInteger_MV &
                        ( Multiplier, Multiplicand ) result ( P_I_MV )
    
    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "Multiplier * Multiplicand" 

    integer ( KDI ), intent ( in ) :: &
      Multiplier
    class ( MeasuredValueForm ), intent ( in ) :: &
      Multiplicand
    type ( MeasuredValueForm ) :: &
      P_I_MV
    
    P_I_MV % Number = real ( Multiplier, KDR ) * Multiplicand % Number
    P_I_MV % Unit   = Multiplicand % Unit
    P_I_MV % Label  = Multiplicand % Label
    
  end function ProductInteger_MV
  
  
  elemental function Power_MV_Integer ( Base, Exponent ) result ( P_MV_I )
    
    class ( MeasuredValueForm ), intent ( in ) :: &
      Base
    integer ( KDI ), intent ( in ) :: &
      Exponent
    type ( MeasuredValueForm ) :: &
      P_MV_I
    
    P_MV_I % Number = Base % Number ** Exponent

    P_MV_I % Unit = P_MV_I % PowerUnit ( Base % Unit, Exponent )

    P_MV_I % Label = P_MV_I % PowerUnit ( Base % Label, Exponent )

  end function Power_MV_Integer
  

  elemental function Power_MV_Real ( Base, Exponent ) result ( P_MV_R )
    
    class ( MeasuredValueForm ), intent ( in ) :: &
      Base
    real ( KDR ), intent ( in ) :: &
      Exponent
    type ( MeasuredValueForm ) :: &
      P_MV_R
    
    P_MV_R % Number = Base % Number ** Exponent
    
    P_MV_R % Unit = P_MV_R % PowerUnit ( Base % Unit, Exponent )
    
    P_MV_R % Label = P_MV_R % PowerUnit ( Base % Label, Exponent )
    
  end function Power_MV_Real
  

  elemental function Quotient_MV_MV ( Dividend, Divisor ) result ( Q_MV_MV )
    
    class ( MeasuredValueForm ), intent ( in ) :: &
      Dividend, &
      Divisor
    type ( MeasuredValueForm ) :: &
      Q_MV_MV
    
    Q_MV_MV = Dividend * Divisor ** ( -1 )

  end function Quotient_MV_MV
  
  
  elemental function Quotient_MV_Real ( Dividend, Divisor ) result ( Q_MV_R )
    
    class ( MeasuredValueForm ), intent ( in ) :: &
      Dividend
    real ( KDR ), intent ( in ) :: &
      Divisor
    type ( MeasuredValueForm ) :: &
      Q_MV_R
    
    Q_MV_R = Dividend * Divisor ** ( -1 )

  end function Quotient_MV_Real
  
  
  elemental function QuotientReal_MV ( Dividend, Divisor ) result ( Q_R_MV )
    
    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "Dividend / Divisor" 

    real ( KDR ), intent ( in ) :: &
      Dividend
    class ( MeasuredValueForm ), intent ( in ) :: &
      Divisor
    type ( MeasuredValueForm ) :: &
      Q_R_MV
 
    Q_R_MV = Dividend * Divisor ** ( -1 )

  end function QuotientReal_MV
  
  
  elemental function Quotient_MV_Integer ( Dividend, Divisor ) result ( Q_MV_I )
    
    class ( MeasuredValueForm ), intent ( in ) :: &
      Dividend
    integer ( KDI ), intent ( in ) :: &
      Divisor
    type ( MeasuredValueForm ) :: &
      Q_MV_I
    
    Q_MV_I = Dividend * real ( Divisor, KDR ) ** ( -1 )

  end function Quotient_MV_Integer
  
  
  elemental function QuotientInteger_MV ( Dividend, Divisor ) result ( Q_I_MV )
    
    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "Dividend / Divisor" 

    integer ( KDI ), intent ( in ) :: &
      Dividend
    class ( MeasuredValueForm ), intent ( in ) :: &
      Divisor
    type ( MeasuredValueForm ) :: &
      Q_I_MV
 
    Q_I_MV = real ( Dividend, KDR ) * Divisor ** ( -1 )

  end function QuotientInteger_MV
  
  
  elemental subroutine AssignReal_MV ( A, MV )

    real ( KDR ), intent ( inout ) :: &
      A
    class ( MeasuredValueForm ), intent ( in ) :: &
      MV

    A = MV % Number

  end subroutine AssignReal_MV


  elemental function EqualTo_MV_MV ( MV_1, MV_2 ) result ( ET )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV_1, MV_2
    logical ( KDL ) :: &
      ET

    ET = .false.

    if ( MV_1 % Unit == MV_2 % Unit &
         .and. MV_1 % Number == MV_2 % Number ) &
      ET = .true.

  end function EqualTo_MV_MV


  elemental function EqualTo_MV_Real ( MV, A ) result ( ET )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    real ( KDR ), intent ( in ) :: &
      A
    logical ( KDL ) :: &
      ET

    ET = .false.

    if ( MV % Number == A ) ET = .true.

  end function EqualTo_MV_Real


  elemental function EqualToReal_MV ( A, MV ) result ( ET )

    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "A == MV" 

    real ( KDR ), intent ( in ) :: &
      A
    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    logical ( KDL ) :: &
      ET

    ET = .false.

    if ( A == MV % Number ) ET = .true.

  end function EqualToReal_MV


  elemental function EqualTo_MV_Integer ( MV, A ) result ( ET )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    integer ( KDI ), intent ( in ) :: &
      A
    logical ( KDL ) :: &
      ET

    ET = .false.

    if ( MV % Number == real ( A, KDR ) ) ET = .true.

  end function EqualTo_MV_Integer


  elemental function EqualToInteger_MV ( A, MV ) result ( ET )

    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "A == MV" 

    integer ( KDI ), intent ( in ) :: &
      A
    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    logical ( KDL ) :: &
      ET

    ET = .false.

    if ( real ( A, KDR ) == MV % Number ) ET = .true.

  end function EqualToInteger_MV


  elemental function NotEqualTo_MV_MV ( MV_1, MV_2 ) result ( NET )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV_1, MV_2
    logical ( KDL ) :: &
      NET

    NET = .false.

    if ( MV_1 % Unit == MV_2 % Unit &
         .and. MV_1 % Number /= MV_2 % Number ) &
      NET = .true.

  end function NotEqualTo_MV_MV


  elemental function NotEqualTo_MV_Real ( MV, A ) result ( NET )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    real ( KDR ), intent ( in ) :: &
      A
    logical ( KDL ) :: &
      NET

    NET = .false.

    if ( MV % Number /= A ) NET = .true.

  end function NotEqualTo_MV_Real


  elemental function NotEqualToReal_MV ( A, MV ) result ( NET )

    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "A /= MV" 

    real ( KDR ), intent ( in ) :: &
      A
    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    logical ( KDL ) :: &
      NET

    NET = .false.

    if ( A /= MV % Number ) NET = .true.

  end function NotEqualToReal_MV


  elemental function NotEqualTo_MV_Integer ( MV, A ) result ( NET )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    integer ( KDI ), intent ( in ) :: &
      A
    logical ( KDL ) :: &
      NET

    NET = .false.

    if ( MV % Number /= real ( A, KDR ) ) NET = .true.

  end function NotEqualTo_MV_Integer


  elemental function NotEqualToInteger_MV ( A, MV ) result ( NET )

    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "A /= MV" 

    integer ( KDI ), intent ( in ) :: &
      A
    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    logical ( KDL ) :: &
      NET

    NET = .false.

    if ( real ( A, KDR ) /= MV % Number ) NET = .true.

  end function NotEqualToInteger_MV


  elemental function GreaterThan_MV_MV ( MV_1, MV_2 ) result ( GT )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV_1, MV_2
    logical ( KDL ) :: &
      GT

    GT = .false.

    if ( MV_1 % Unit == MV_2 % Unit &
         .and. MV_1 % Number > MV_2 % Number ) &
      GT = .true.

  end function GreaterThan_MV_MV


  elemental function GreaterThan_MV_Real ( MV, A ) result ( GT )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    real ( KDR ), intent ( in ) :: &
      A
    logical ( KDL ) :: &
      GT

    GT = .false.

    if ( MV % Number > A ) GT = .true.

  end function GreaterThan_MV_Real


  elemental function GreaterThanReal_MV ( A, MV ) result ( GT )

    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "A > MV" 

    real ( KDR ), intent ( in ) :: &
      A
    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    logical ( KDL ) :: &
      GT

    GT = .false.

    if ( A > MV % Number ) GT = .true.

  end function GreaterThanReal_MV


  elemental function GreaterThan_MV_Integer ( MV, A ) result ( GT )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    integer ( KDI ), intent ( in ) :: &
      A
    logical ( KDL ) :: &
      GT

    GT = .false.

    if ( MV % Number > real ( A, KDR ) ) GT = .true.

  end function GreaterThan_MV_Integer


  elemental function GreaterThanInteger_MV ( A, MV ) result ( GT )

    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "A > MV" 

    integer ( KDI ), intent ( in ) :: &
      A
    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    logical ( KDL ) :: &
      GT

    GT = .false.

    if ( real ( A, KDR ) > MV % Number ) GT = .true.

  end function GreaterThanInteger_MV


  elemental function LessThan_MV_MV ( MV_1, MV_2 ) result ( LT )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV_1, MV_2
    logical ( KDL ) :: &
      LT

    LT = .false.

    if ( MV_1 % Unit == MV_2 % Unit &
         .and. MV_1 % Number < MV_2 % Number ) &
      LT = .true.

  end function LessThan_MV_MV


  elemental function LessThan_MV_Real ( MV, A ) result ( LT )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    real ( KDR ), intent ( in ) :: &
      A
    logical ( KDL ) :: &
      LT

    LT = .false.

    if ( MV % Number < A ) LT = .true.

  end function LessThan_MV_Real


  elemental function LessThanReal_MV ( A, MV ) result ( LT )

    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "A < MV" 

    real ( KDR ), intent ( in ) :: &
      A
    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    logical ( KDL ) :: &
      LT

    LT = .false.

    if ( A < MV % Number ) LT = .true.

  end function LessThanReal_MV


  elemental function LessThan_MV_Integer ( MV, A ) result ( LT )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    integer ( KDI ), intent ( in ) :: &
      A
    logical ( KDL ) :: &
      LT

    LT = .false.

    if ( MV % Number < real ( A, KDR ) ) LT = .true.

  end function LessThan_MV_Integer


  elemental function LessThanInteger_MV ( A, MV ) result ( LT )

    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "A < MV" 

    integer ( KDI ), intent ( in ) :: &
      A
    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    logical ( KDL ) :: &
      LT

    LT = .false.

    if ( real ( A, KDR ) < MV % Number ) LT = .true.

  end function LessThanInteger_MV


  elemental function GreaterThanEqualTo_MV_MV ( MV_1, MV_2 ) result ( GTE )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV_1, MV_2
    logical ( KDL ) :: &
      GTE

    GTE = .false.

    if ( MV_1 % Unit == MV_2 % Unit &
         .and. MV_1 % Number >= MV_2 % Number ) &
      GTE = .true.

  end function GreaterThanEqualTo_MV_MV


  elemental function GreaterThanEqualTo_MV_Real ( MV, A ) result ( GTE )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    real ( KDR ), intent ( in ) :: &
      A
    logical ( KDL ) :: &
      GTE

    GTE = .false.

    if ( MV % Number >= A ) GTE = .true.

  end function GreaterThanEqualTo_MV_Real


  elemental function GreaterThanEqualToReal_MV ( A, MV ) result ( GTE )

    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "A >= MV" 

    real ( KDR ), intent ( in ) :: &
      A
    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    logical ( KDL ) :: &
      GTE

    GTE = .false.

    if ( A >= MV % Number ) GTE = .true.

  end function GreaterThanEqualToReal_MV


  elemental function GreaterThanEqualTo_MV_Integer ( MV, A ) result ( GTE )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    integer ( KDI ), intent ( in ) :: &
      A
    logical ( KDL ) :: &
      GTE

    GTE = .false.

    if ( MV % Number >= real ( A, KDR ) ) GTE = .true.

  end function GreaterThanEqualTo_MV_Integer


  elemental function GreaterThanEqualToInteger_MV ( A, MV ) result ( GTE )

    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "A >= MV" 

    integer ( KDI ), intent ( in ) :: &
      A
    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    logical ( KDL ) :: &
      GTE

    GTE = .false.

    if ( real ( A, KDR ) >= MV % Number ) GTE = .true.

  end function GreaterThanEqualToInteger_MV


  elemental function LessThanEqualTo_MV_MV ( MV_1, MV_2 ) result ( LTE )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV_1, MV_2
    logical ( KDL ) :: &
      LTE

    LTE = .false.

    if ( MV_1 % Unit == MV_2 % Unit &
         .and. MV_1 % Number <= MV_2 % Number ) &
      LTE = .true.

  end function LessThanEqualTo_MV_MV


  elemental function LessThanEqualTo_MV_Real ( MV, A ) result ( LTE )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    real ( KDR ), intent ( in ) :: &
      A
    logical ( KDL ) :: &
      LTE

    LTE = .false.

    if ( MV % Number <= A ) LTE = .true.

  end function LessThanEqualTo_MV_Real


  elemental function LessThanEqualToReal_MV ( A, MV ) result ( LTE )

    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "A <= MV" 

    real ( KDR ), intent ( in ) :: &
      A
    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    logical ( KDL ) :: &
      LTE

    LTE = .false.

    if ( A <= MV % Number ) LTE = .true.

  end function LessThanEqualToReal_MV


  elemental function LessThanEqualTo_MV_Integer ( MV, A ) result ( LTE )

    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    integer ( KDI ), intent ( in ) :: &
      A
    logical ( KDL ) :: &
      LTE

    LTE = .false.

    if ( MV % Number <= real ( A, KDR ) ) LTE = .true.

  end function LessThanEqualTo_MV_Integer


  elemental function LessThanEqualToInteger_MV ( A, MV ) result ( LTE )

    !-- Convention on argument ordering is broken here to maintain 
    !   correspondence with order of entities in the expression
    !   "A <= MV" 

    integer ( KDI ), intent ( in ) :: &
      A
    class ( MeasuredValueForm ), intent ( in ) :: &
      MV
    logical ( KDL ) :: &
      LTE

    LTE = .false.

    if ( real ( A, KDR ) <= MV % Number ) LTE = .true.

  end function LessThanEqualToInteger_MV


  elemental function ProductUnit ( Unit_1, Unit_2 ) result ( P_U )

    character ( * ), intent ( in ) :: &
      Unit_1, &
      Unit_2
    character ( LDL ) :: &
      P_U

    integer ( KDI ) :: &
      Caret, &
      Dot, &
      Exponent_1, &
      Exponent_2, &
      Exponent
    character ( LDL ) :: &
      UnitBase_1, &
      UnitBase_2, &
      ExponentString
    character ( LDL ), dimension ( : ), allocatable :: &
      UnitPiece_1, &
      UnitPiece_2
  
    P_U = ''

    if ( trim ( Unit_1 ) == '' .and. trim ( Unit_2 ) == '' ) return

    call Split ( Unit_1, ' ', UnitPiece_1 )
    call Split ( Unit_2, ' ', UnitPiece_2 )
    if ( size ( UnitPiece_1 ) > 1 .or. size ( UnitPiece_2 ) > 1 ) then
      !-- Give up on this more complicated case
      P_U = adjustl ( trim ( Unit_1 ) // ' ' // trim ( Unit_2 ) )
      return
    end if

    !-- Analyze Unit_1
    Caret = index ( Unit_1, '^' )     
    if ( Caret > 0 ) then
      UnitBase_1 = Unit_1 ( : Caret - 1 )
      Dot = index ( Unit_1 ( Caret + 1 : ), '.' )
      if ( Dot > 0 ) then
!        print *, 'ERROR: Real exponents not implemented in Product_MV_MV'
!        stop
      else
        read ( Unit_1 ( Caret + 1 : ), fmt = '(i7)') Exponent_1
      end if
    else
      UnitBase_1 = Unit_1
      Exponent_1 = 1
    end if

    !-- Analyze Unit_2
    Caret = index ( Unit_2, '^' )     
    if ( Caret > 0 ) then
      UnitBase_2 = Unit_2 ( : Caret - 1 )
      Dot = index ( Unit_2 ( Caret + 1 : ), '.' )
      if ( Dot > 0 ) then
!        print *, 'ERROR: Real exponents not implemented in Product_MV_MV'
!        stop
      else
        read ( Unit_2 ( Caret + 1 : ), fmt = '(i7)') Exponent_2
      end if
    else
      UnitBase_2 = Unit_2
      Exponent_2 = 1
    end if

    if ( trim ( UnitBase_1 ) == trim ( UnitBase_2 ) ) then
      Exponent = Exponent_1 + Exponent_2
      if ( Exponent == 0 ) then
        P_U = ''
      else if ( Exponent == 1 ) then
        P_U = trim ( UnitBase_1 )
      else 
        write ( ExponentString, fmt ='(i7)' ) Exponent
        P_U = trim ( UnitBase_1 ) // '^' // trim ( adjustl ( ExponentString ) )
      end if
    else
      P_U = adjustl ( trim ( Unit_1 ) // ' ' // trim ( Unit_2 ) )
    end if

! print*, 'UnitBase_1 ', trim ( UnitBase_1 )
! print*, 'UnitBase_2 ', trim ( UnitBase_2 )
! print*, 'Exponent_1 ', Exponent_1
! print*, 'Exponent_2 ', Exponent_2
! print*, 'Exponent', Exponent

  end function ProductUnit


  elemental function PowerUnitInteger ( Unit, Exponent ) result ( P_U )

    character ( * ), intent ( in ) :: &
      Unit
    integer ( KDI ), intent ( in ) :: &
      Exponent
    character ( LDL ) :: &
      P_U
    
    integer ( KDI ) :: &
      iU, &  !-- iUnit
      Caret, &
      Dot, &
      OldIntegerExponent
    real ( KDR ) :: &
      OldRealExponent
    character ( LDL ) :: &
      ExponentString, &
      Scratch
    character ( LDL ), dimension ( : ), allocatable :: &
      UnitPiece
      
    P_U = ''

    if ( Unit == '' ) return

    call Split ( Unit, ' ', UnitPiece )
    
    do iU = 1, size ( UnitPiece )

      Scratch = UnitPiece ( iU )
      Caret = index ( Scratch, '^' )
      
      if ( Caret > 0 ) then
        Dot = index ( Scratch ( Caret + 1 : ), '.' )
        if ( Dot > 0 ) then
          read ( Scratch ( Caret + 1 : ), fmt = '(f7.4)' ) OldRealExponent
          write &
            ( ExponentString, fmt = '(f7.4)' )  ( OldRealExponent * Exponent )
        else
          read ( Scratch ( Caret + 1 : ), fmt = '(i7)') &
               OldIntegerExponent
          write &
            ( ExponentString, fmt ='(i7)' ) ( OldIntegerExponent *  Exponent )
        end if
        UnitPiece ( iU ) &
          = trim ( Scratch ( : Caret ) ) // trim ( adjustl ( ExponentString ) )
      else
        if ( Exponent == 1 ) exit
        write ( ExponentString, fmt = '(i7)' ) Exponent
        UnitPiece ( iU ) = &
          trim ( Scratch ) // '^' // trim ( adjustl ( ExponentString ) )
      end if

    end do 

    call Join ( UnitPiece, ' ', P_U )
      
  end function PowerUnitInteger


  elemental function PowerUnitReal ( Unit, Exponent ) result ( P_U )

    character ( * ), intent ( in ) :: &
      Unit
    real ( KDR ), intent ( in ) :: &
      Exponent
    character ( LDL ) :: &
      P_U
    
    integer ( KDI ) :: &
      iU, &  !-- iUnit
      Caret, &
      Dot, &
      OldIntegerExponent
    real ( KDR ) :: &
      OldRealExponent
    character ( LDL ) :: &
      ExponentString, &
      Scratch
    character ( LDL ), dimension ( : ), allocatable :: &
      UnitPiece
      
    P_U = ''

    if ( Unit == '' ) return

    call Split ( Unit, ' ', UnitPiece )
    
    do iU = 1, size ( UnitPiece )

      Scratch = UnitPiece ( iU )
      Caret = index ( Scratch, '^' )
     
      if ( Caret > 0 ) then
        Dot = index ( Scratch ( Caret + 1 : ), '.' )
        if ( Dot > 0 ) then
          read ( Scratch ( Caret + 1 : ), fmt = '(f7.4)' ) OldRealExponent
          write &
            ( ExponentString, fmt = '(f7.4)' )  ( OldRealExponent * Exponent )
        else
          read ( Scratch ( Caret + 1 : ), fmt = '(i7)') &
               OldIntegerExponent
          write &
            ( ExponentString, fmt ='(f7.4)' ) ( OldIntegerExponent * Exponent )
        end if
        if ( trim ( adjustl ( ExponentString ) ) == '1.0000' ) then
          UnitPiece ( iU ) = trim ( Scratch ( : Caret - 1 ) )
        else
          UnitPiece ( iU ) &
            = trim ( Scratch ( : Caret ) ) // trim ( adjustl ( ExponentString ) )
        end if
      else
        write ( ExponentString, fmt = '(f7.4)' ) Exponent
        if ( trim ( adjustl ( ExponentString ) ) == '1.0000' ) then
          UnitPiece ( iU ) = trim ( Scratch )
        else
          UnitPiece ( iU ) = &
            trim ( Scratch ) // '^' // trim ( adjustl ( ExponentString ) )
        end if
      end if

    end do
    
    call Join ( UnitPiece, ' ', P_U )
      
  end function PowerUnitReal


end module MeasuredValue_Form
